@@ -481,7 +481,7 @@ function wp_http_validate_url( $url ) {
 		}
 		if ( $ip ) {
 			$parts = array_map( 'intval', explode( '.', $ip ) );
-			if ( 127 === $parts[0] || 10 === $parts[0]
+			if ( 127 === $parts[0] || 10 === $parts[0] || 0 === $parts[0]
 				|| ( 172 === $parts[0] && 16 <= $parts[1] && 31 >= $parts[1] )
 				|| ( 192 === $parts[0] && 168 === $parts[1] )
 			) {
@@ -545,11 +545,11 @@ function allowed_http_request_hosts( $is_external, $host ) {
  * @return bool
  */
 function ms_allowed_http_request_hosts( $is_external, $host ) {
-	global $wpdb, $current_site;
+	global $wpdb;
 	static $queried = array();
 	if ( $is_external )
 		return $is_external;
-	if ( $host === $current_site->domain )
+	if ( $host === get_current_site()->domain )
 		return true;
 	if ( isset( $queried[ $host ] ) )
 		return $queried[ $host ];
